==> eventbus-0.0.14.tgz: package/lib/Bus.js <==
"use strict";

const EventEmitter = require('events');
const Promise      = require('bluebird');
const debug        = require('debug')('eventbus:bus');
const amqplib      = require('amqplib');
const isFatalError = require('amqplib/lib/connection').isFatalError;
const domain       = require('domain');

const RECONNET_INTERVAL_DEFAULT = 1; // in seconds

class Bus extends EventEmitter {

    constructor(microServiceName, options) {
        super();
        this.url               = options.url;
        this.microServiceName  = microServiceName;
        this.reconnectInterval = (options.reconnectInterval || RECONNET_INTERVAL_DEFAULT) * 1000;
    }

    _createErrorHandler() {
        if (this.errorHandlerCreated) {
            return;
        }

        this.errorHandlerCreated = true;
        this.on('error', (err) => {
            if (isFatalError(err)) { // reconnect only on fatal error
                this.reconnect = true;
                debug(`reconnecting in: ${this.reconnectInterval / 1000} seconds because of fatal error`);
                setTimeout(() => {
                    debug(`reconnecting`);
                    this.init();
                }, this.reconnectInterval);
            }
        });
    }

    async quit() {
        await this.connection.close();
        debug('bus closed');
    }

    init() {
        this._createErrorHandler();

        this.openPromise = amqplib.connect(this.url);
        this.initialized = true;
        this.openPromise
            .then((newConnection) => {
                debug('new connection created');
                if (this.reconnect && this.connection) {
                    this.publishChannel = undefined;
                    debug('reconnected');
                }
                this.reconnect  = false;
                this.connection = newConnection;

                debug(`ready`);
                this.emit('ready');

                this.connection.on('error', (err) => {
                    this.emit('error', err);
                });

                this.connection.on('close', () => {
                    debug(`connection close event`);
                    this.emit('close');
                });

                this.connection.on('blocked', () => {
                    debug('connection blocked');
                });

                this.connection.on('unblocked', () => {
                    debug('connection unblocked');
                });
            })
            .catch((err) => {
                this.emit('error', err);
            })
            .done();
    }

    publish(eventName, msg) {
        return Promise.resolve()
            .then(() => {
                if (!this.publishChannel) {
                    return this._createPublishChannel();
                }
            })
            .then(() => {
                return this.publishChannel.assertExchange(eventName, 'fanout', { durable: true });
            })
            .then(() => {
                return this.publishChannel.publish(eventName, 'my-routing-key', new Buffer(this._serialize(msg)));
            });
    }

    subscribe(eventName, handler) {
        const subscriberEmitter = new EventEmitter();
        subscriberEmitter.on('error', () => {});

        const queueName = `${this.microServiceName}_${eventName}`;
        Promise.resolve()
            .then(() => {
                return this._getSubscribeChannel();
            })
            .then((ch) => {
                return ch.assertExchange(eventName, 'fanout', { durable: true })
                    .then(() => {
                        return ch.assertQueue(queueName, { durable: true })
                            .then(() => {
                                debug(`queue: ${queueName} asserted`);
                                return ch.bindQueue(queueName, eventName);
                            })
                            .then(() => {
                                return ch.consume(queueName, (msg) => {
                                    const d = domain.create();
                                    d.on('error', (err) => {
                                        subscriberEmitter.emit('error', err);
                                    });
                                    d.run(() => {
                                        ch.ack(msg); // TODO allow controlling when to ack
                                        handler(this._deserialize(msg.content.toString()))
                                            .catch((err) => {
                                                subscriberEmitter.emit('error', err);
                                            });
                                    });
                                });
                            });
                    });
            })
            .catch((err) => {
                subscriberEmitter.emit('error', { connection: err });
            })
            .done();

        return subscriberEmitter;
    }

    _createPublishChannel() {
        return this.openPromise
            .then((conn) => {
                return conn.createChannel();
            })
            .then((ch) => {
                this.publishChannel = ch;
            });
    }

    _getSubscribeChannel() {
        return this.openPromise
            .then((conn) => {
                return conn.createChannel();
            });
    }

    _serialize(body) {
        return JSON.stringify(body);
    }

    _deserialize(content) {
        return JSON.parse(content);
    }


}


module.exports = Bus;

==> eventbus-0.0.23.tgz: package/lib/Bus.js <==
"use strict";

const EventEmitter = require('events');
const Promise      = require('bluebird');
const debug        = require('debug')('eventbus:bus');
const amqplib      = require('amqplib');
const isFatalError = require('amqplib/lib/connection').isFatalError;
const domain       = require('domain');

const RECONNET_INTERVAL_DEFAULT = 1; // in seconds

class Bus extends EventEmitter {

    constructor(microServiceName, options) {
        super();
        this.url               = options.url;
        this.microServiceName  = microServiceName;
        this.reconnectInterval = (options.reconnectInterval || RECONNET_INTERVAL_DEFAULT) * 1000;
    }

    _createErrorHandler() {
        if (this.errorHandlerCreated) {
            return;
        }

        this.errorHandlerCreated = true;
        this.on('error', (err) => {
            if (isFatalError(err)) { // reconnect only on fatal error
                this.reconnect = true;
                debug(`reconnecting in: ${this.reconnectInterval / 1000} seconds because of fatal error`);
                setTimeout(() => {
                    debug(`reconnecting`);
                    this.init();
                }, this.reconnectInterval);
            }
        });
    }

    async quit() {
        await this.connection.close();
        debug('bus closed');
    }

    init() {
        this._createErrorHandler();

        this.openPromise = amqplib.connect(this.url);
        this.initialized = true;
        this.openPromise
            .then((newConnection) => {
                debug('new connection created');
                if (this.reconnect && this.connection) {
                    this.publishChannel = undefined;
                    debug('reconnected');
                }
                this.reconnect  = false;
                this.connection = newConnection;

                debug(`ready`);
                this.emit('ready');

                this.connection.on('error', (err) => {
                    this.emit('error', err);
                });

                this.connection.on('close', () => {
                    debug(`connection close event`);
                    this.emit('close');
                });

                this.connection.on('blocked', () => {
                    debug('connection blocked');
                });

                this.connection.on('unblocked', () => {
                    debug('connection unblocked');
                });
            })
            .catch((err) => {
                this.emit('error', err);
            })
            .done();
    }

    publish(eventName, msg) {
        return Promise.resolve()
            .then(() => {
                if (!this.publishChannel) {
                    return this._createPublishChannel();
                }
            })
            .then(() => {
                return this.publishChannel.assertExchange(eventName, 'fanout', { durable: true });
            })
            .then(() => {
                return this.publishChannel.publish(eventName, 'my-routing-key', new Buffer(this._serialize(msg)));
            });
    }

    subscribe(eventName, handler, options = {}) {
        const subscriberEmitter = new EventEmitter();
        subscriberEmitter.on('error', () => {});
        const waitForAck = options.waitForAck || false;
        const concurrencyLimit =  options.concurrencyLimit || false;
        const messageTtl = options.ttl || false;

        const queueName = `${this.microServiceName}_${eventName}`;
        Promise.resolve()
            .then(() => {
                return this._getSubscribeChannel();
            })
            .then((ch) => {
                return ch.assertExchange(eventName, 'fanout', { durable: true })
                    .then(() => {
                        const queueOptions = { durable: true };
                        if (messageTtl) {
                            queueOptions.messageTtl = messageTtl;
                        }
                        return ch.assertQueue(queueName, queueOptions)
                            .then(() => {
                                debug(`queue: ${queueName} asserted`);
                                return ch.bindQueue(queueName, eventName);
                            })
                            .then(() => {
                                return ch.prefetch(concurrencyLimit);
                            })
                            .then(() => {
                                debug(`queue: ${queueName} concurrencyLimit: ${concurrencyLimit} was set`);
                                return ch.consume(queueName, (msg) => {
                                    const d = domain.create();
                                    d.on('error', (err) => {
                                        subscriberEmitter.emit('error', err);
                                    });
                                    d.run(() => {
                                        if (!waitForAck) {
                                            ch.ack(msg);
                                        }
                                        handler(this._deserialize(msg.content.toString()))
                                            .then(() => {
                                                if (waitForAck) {
                                                    ch.ack(msg);
                                                }
                                            })
                                            .catch((err) => {
                                                subscriberEmitter.emit('error', err);
                                                if (waitForAck) {
                                                    ch.nack(msg);
                                                }
                                            });
                                    });
                                });
                            });
                    });
            })
            .catch((err) => {
                subscriberEmitter.emit('error', { connection: err });
            })
            .done();

        return subscriberEmitter;
    }

    _createPublishChannel() {
        return this.openPromise
            .then((conn) => {
                return conn.createChannel();
            })
            .then((ch) => {
                this.publishChannel = ch;
            });
    }

    _getSubscribeChannel() {
        return this.openPromise
            .then((conn) => {
                return conn.createChannel();
            });
    }

    _serialize(body) {
        return JSON.stringify(body);
    }

    _deserialize(content) {
        return JSON.parse(content);
    }


}


module.exports = Bus;
